mode(1)
//
// Demo of assert_close.sci
//

// Comparisons which are successful.
// Relative error :
assert_close ( 1.0 , 1.0 , %eps );
assert_close ( ones(10,1) , ones(10,1) , %eps );
// Absolute error :
assert_close ( 1.23456789123456789e-30 , 0 , 1.e-10 );
assert_close ( [1.0 %nan], [1.0 %nan] , %eps );
halt()   // Press return to continue
 
// Comparisons which are failures.
// Error message produced :
assert_close ( 1.0 , 2.0 , %eps );
// Error message produced :
flag = assert_close ( 1.0 , 2.0 , %eps )
// No error message produced :
[flag,errmsg] = assert_close ( 1.0 , 2.0 , %eps )
assert_close ( 1 , [2 3] , %eps );
assert_close ( [%nan 1.0], [1.0 %nan] , %eps );
assert_close ( 1.0 + 5 * %eps , 1.0 , %eps );
assert_close ( 1.23456789123456789e-30 , 1.3e-30 , %eps );
halt()   // Press return to continue
 
// In the case where expected is nonzero, the
// tolerance for relative error should be a
// multiple of %eps.
// The following test is a success and shows
// that less than 11 digits are lost with respect
// to the maximum possible accuracy.
assert_close ( 1.23456 , 1.23457 , 1.e11*%eps );
halt()   // Press return to continue
 
// We must not exchange computed and expected.
// The following test pass: we use an absolute tolerance
// because the expected value is zero.
assert_close ( 1.23456789e-30 , 0 , 1.e-10 );
// The following test fail: we use a relative tolerance
// because the expected value is nonzero.
assert_close ( 0 , 1.23456789e-30 , 1.e-10 );
halt()   // Press return to continue
 
// We must configure the precision as tightly as possible.
// The following test fails, because the precision is too small
// with respect to the given numbers.
assert_close ( 1.23456 , 1.23457 , %eps );
// In order to get the number of lost digits,
// we can use the following formula :
ceil(log10(norm(computed-expected)/norm(expected)/%eps))
// which returns 11.
// We now make a judgment on the accuracy and conclude
// that 11 digits lost is acceptable: the following test pass.
assert_close ( 1.23456 , 1.23457 , 1.e11*%eps );
halt()   // Press return to continue
 
// We give here a practical example of the use of assert_close.
// We solve an averagely ill-conditionned system of linear
// equations, based on the Hilbert matrix.
n = 6;
// The expected value is set by ourselves.
expected = ones(n,1);
A = testmatrix("hilb",n);
// We compute the condition number of the matrix : ~10^8
ceil(log10(cond(A)))
// This means that the number of digits lost,
// predicted by theory, is 8.
// The right-hand side is computed given A and expected.
b = A * expected;
// In this case, a Gauss algorithm with partial
// pivoting is used.
computed = A\b;
// The following test fails: we have lost some digits.
assert_close(computed,expected,%eps)
// We compute the actual number of lost digits: 5
ceil(log10(norm(computed-expected)/norm(expected)/%eps))
// We accept this computation, since the actual number of lost
// digits (i.e. 5) is close - but lower - than what was
// predicted by theory (i.e. 8).
// The following test pass.
assert_close(computed,expected,1.e5*%eps);
halt()   // Press return to continue
 
//========= E N D === O F === D E M O =========//
//
// Load this script into the editor
//
filename = "assert_close.sce";
dname = get_absolute_file_path(filename);
editor ( dname + filename );
