// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) 2008 - INRIA - Michael Baudin
// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt


function flag = MY_assert_equal ( computed , expected )
  if computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

function checkassert ( flag , errmsg , ctype )
  if ( ctype == "success" ) then
    MY_assert_equal ( (flag==%t) & (errmsg==""), %t )
  else
    MY_assert_equal ( (flag==%f) & (errmsg<>""), %t )
  end
endfunction
    

format("v",10);

// Check error message when number of arguments is false
instr = "assert_close ( )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 10000 );
//
instr = "assert_close ( 1 )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 10000 );
//
instr = "assert_close ( 1 , 1 )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 10000 );
//
instr = "[o1,o2,o3]=assert_close ( 1 , 1 , %eps )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 59 );
//
//////////////////////////////////////////
// Check error message when type of arguments is false
instr = "assert_close ( ""a"" , 2 , %eps )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 10000 );
//
instr = "assert_close ( 1 , ""b"" , %eps )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 10000 );
//
instr = "assert_close ( 1 , 2 , ""c"" )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 10000 );
//////////////////////////////////////////
// Check error message when size of arguments is false
instr = "assert_close ( 1 , 2 , [1 1] )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 10000 );
//
instr = "assert_close ( 1 , [2 3] , %eps )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 10000 );

//////////////////////////////////////////
//
// Check that the error message is correctly handled.
instr = "assert_close ( 1.0 , 2.0 , %eps )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 10000 );
errmsg = lasterror();
MY_assert_equal ( errmsg , "assert_close: Assertion failed: expected = 2 while computed = 1 with precision=2.220D-16" );
//
// Check that the error message is correctly handled.
instr = "assert_close ( 1.0 , 1.0 , %eps )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 0 );
errmsg = lasterror();
MY_assert_equal ( errmsg , [] );
//////////////////////////////////////////
//
//
// Obvious failure
// Check the error message
instr = "assert_close ( zeros(10,1)+1.e-4 , zeros(10,1) , 1.e-5 )";
ierr=execstr(instr,"errcatch");
MY_assert_equal ( ierr , 10000 );
errmsg = lasterror();
MY_assert_equal ( errmsg , "assert_close: Assertion failed: expected = [0 ...] while computed = [0.0001 ...] with precision=0.00001" );
//
// Obvious success
[flag,errmsg] = assert_close ( 1.0 , 1.0 , %eps );
checkassert ( flag , errmsg , "success" );
//
// Obvious success
[flag,errmsg] = assert_close ( ones(10,1) , ones(10,1) , %eps );
checkassert ( flag , errmsg , "success" );
//
// Obvious success
[flag,errmsg] = assert_close ( ones(10,1)+%eps , ones(10,1) , %eps );
checkassert ( flag , errmsg , "success" );
//
// Obvious success
[flag,errmsg] = assert_close ( zeros(10,1) , zeros(10,1) , 1.e-5 );
checkassert ( flag , errmsg , "success" );
//
// Obvious failure
[flag,errmsg] = assert_close ( 1.0 , 2.0 , %eps );
checkassert ( flag , errmsg , "failure" );
//
// Obvious failure
[flag,errmsg] = assert_close ( zeros(10,1)+1.e-4 , zeros(10,1) , 1.e-5 );
checkassert ( flag , errmsg , "failure" );
//
// Success: not obvious!
// The two values are equal, very small, but nonzero.
// The relative tolerance must be used here.
// If, instead, a bug in the assert function is so that the 
// absolute tolerance is used as 10^-16, then the output 
// of this comparison is wrong.
[flag,errmsg] = assert_close ( 1.23456789123456789e-30 , 1.23456789123456789e-30 , %eps );
checkassert ( flag , errmsg , "success" );
//
// Failure : not obvious!
// There is only one significant digit here and we require the 
// maximum precision.
// The test must fail because the relative tolerance must be used here.
// If, instead, there is a bug in the comparison and 
// the absolute tolerance is used and set to
// 10^-16, the output of this test is wrong.
[flag,errmsg] = assert_close ( 1.23456789123456789e-30 , 1.3e-30 , %eps );
checkassert ( flag , errmsg , "failure" );
//
// Success : not obvious!
// The expected result is zero, so that the absolute tolerance is used.
[flag,errmsg] = assert_close ( 1.23456789123456789e-30 , 0 , 1.e-10 );
checkassert ( flag , errmsg , "success" );
//
// Check that nans are correctly handled
[flag,errmsg] = assert_close ( [1.0 %nan], [1.0 %nan] , %eps );
checkassert ( flag , errmsg , "success" );
//
// Check that nans are correctly handled
[flag,errmsg] = assert_close ( [1 %nan 1 %nan] , [1 %nan 1 %nan] , %eps );
checkassert ( flag , errmsg , "success" );
//
[flag,errmsg] = assert_close ( [%nan 1.0], [1.0 %nan] , %eps );
checkassert ( flag , errmsg , "failure" );
//
// Check that slightly different values cannot pass basic test
// Simple sequence with default settings
[flag,errmsg] = assert_close ( 1.0 + 5 * %eps , 1.0 , %eps );
checkassert ( flag , errmsg , "failure" );
//
// Customize the relative precision so that a test can pass
// Simple sequence with default settings
[flag,errmsg] = assert_close ( 1.0 + 5 * %eps , 1.0 , 10*%eps );
checkassert ( flag , errmsg , "success" );
//
// Customize the absolute precision so that a test can pass
// Simple sequence with default settings
[flag,errmsg] = assert_close ( 0.0 + 5 * %eps , 0.0 , 10*%eps );
checkassert ( flag , errmsg , "success" );
//
// An example where the relative error is used,
// with a customized tolerance.
[flag,errmsg] = assert_close ( 1.23456 , 1.23457 , 1.e11*%eps );
checkassert ( flag , errmsg , "success" );


