// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) 2008 - INRIA - Michael Baudin
// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt


function flag = MY_assert_equal ( computed , expected )
  if computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

// Compare real values
[order,msg] = assert_comparecomplex ( 1 , -1 , [%eps 0] );
MY_assert_equal ( order , 1 );
MY_assert_equal ( msg<>"" , %t );
//
[order,msg] = assert_comparecomplex ( -1 , 1 , [%eps 0] );
MY_assert_equal ( order , -1 );
MY_assert_equal ( msg<>"" , %t );
//
[order,msg] = assert_comparecomplex ( 1 , 1 , [%eps 0] );
MY_assert_equal ( order , 0 );
MY_assert_equal ( msg<>"" , %t );
//

// Compare complex values #1
[order,msg] = assert_comparecomplex ( 1+2*%i , 1+3*%i , [%eps 0] );
MY_assert_equal ( order , -1 );
MY_assert_equal ( msg<>"" , %t );
//
[order,msg] = assert_comparecomplex ( 1+3*%i , 1+2*%i , [%eps 0] );
MY_assert_equal ( order , 1 );
MY_assert_equal ( msg<>"" , %t );
//
[order,msg] = assert_comparecomplex ( 1+2*%i , 1+2*%i , [%eps 0] );
MY_assert_equal ( order , 0 );
MY_assert_equal ( msg<>"" , %t );
//

// Compare complex values #2
tol = [%eps 0];
[order,msg] = assert_comparecomplex ( 1+%i , -1+%i , tol );
MY_assert_equal ( order , 1 );
MY_assert_equal ( msg<>"" , %t );
//
[order,msg] = assert_comparecomplex ( -1+%i , 1+%i , tol );
MY_assert_equal ( order , -1 );
MY_assert_equal ( msg<>"" , %t );
//
[order,msg] = assert_comparecomplex ( 1+%i , 1+%i , tol );
MY_assert_equal ( order , 0 );
MY_assert_equal ( msg<>"" , %t );
//

// Compare with tolerances : equality cases
[order,msg] = assert_comparecomplex ( 1.2345+%i , 1.2346+%i , [%eps 1.e-3] );
MY_assert_equal ( order , 0 );
MY_assert_equal ( msg<>"" , %t );
//
[order,msg] = assert_comparecomplex ( 1.2345+%i , 1.2346+%i , [1.e12*%eps 0] );
MY_assert_equal ( order , 0 );
MY_assert_equal ( msg<>"" , %t );
//
[order,msg] = assert_comparecomplex ( 1+1.2345*%i , 1+1.2347*%i , [%eps 1.e-3] );
MY_assert_equal ( order , 0 );
MY_assert_equal ( msg<>"" , %t );
//
[order,msg] = assert_comparecomplex ( 1+1.2345*%i , 1+1.2347*%i , [1.e12*%eps 0] );
MY_assert_equal ( order , 0 );
MY_assert_equal ( msg<>"" , %t );
//

// Compare more realistic data
x = [
-0.123452 - 0.123454 * %i
-0.123451 + 0.123453 * %i
0.123458 - 0.123459 * %i
0.123456 + 0.123457 * %i
];
// Consider less than 4 significant digits
tol = [1.e-4 0];
for i = 1 : size(x,"*")-1
  order(i) = assert_comparecomplex ( x(i) , x(i+1) , tol );
end
MY_assert_equal ( order , -ones(3,1) );
clear order;

// Compare data from bug #415
x = [
-1.9914145
-1.895889
-1.6923826
-1.4815461
-1.1302576
-0.5652256 - 0.0655080 * %i
-0.5652256 + 0.0655080 * %i
0.3354023 - 0.1602902 * %i
0.3354023 + 0.1602902 * %i
1.3468911
1.5040136
1.846668
1.9736772
1.9798866
];
// Consider less than 4 significant digits
tol = [1.e-5 0];
for i = 1 : size(x,"*")-1
  order(i) = assert_comparecomplex ( x(i) , x(i+1) , tol );
end
MY_assert_equal ( order , -ones(13,1) );
clear order;

